use crate::prelude::*;

use std::os::raw::c_void;
use std::ptr;

#[repr(C)]
#[derive(Copy, Clone)]
pub struct VkAccelerationStructureGeometryInstancesDataKHR {
    pub sType: VkStructureType,
    pub pNext: *const c_void,
    pub arrayOfPointers: VkBool32,
    pub data: VkDeviceOrHostAddressConstKHR,
}

impl VkAccelerationStructureGeometryInstancesDataKHR {
    #[inline]
    fn new(array_of_pointers: impl Into<VkBool32>, data: VkDeviceOrHostAddressConstKHR) -> Self {
        VkAccelerationStructureGeometryInstancesDataKHR {
            sType: VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_INSTANCES_DATA_KHR,
            pNext: ptr::null(),
            arrayOfPointers: array_of_pointers.into(),
            data,
        }
    }
}

impl<'a> From<&'a [VkAccelerationStructureInstanceKHR]>
    for VkAccelerationStructureGeometryInstancesDataKHR
{
    fn from(instances: &'a [VkAccelerationStructureInstanceKHR]) -> Self {
        Self::new(false, VkDeviceOrHostAddressConstKHR::from(instances))
    }
}

impl<'a> From<&'a [&'a VkAccelerationStructureInstanceKHR]>
    for VkAccelerationStructureGeometryInstancesDataKHR
{
    fn from(instances: &'a [&'a VkAccelerationStructureInstanceKHR]) -> Self {
        Self::new(true, VkDeviceOrHostAddressConstKHR::from(instances))
    }
}

impl From<VkDeviceOrHostAddressConstKHR> for VkAccelerationStructureGeometryInstancesDataKHR {
    fn from(address: VkDeviceOrHostAddressConstKHR) -> Self {
        Self::new(false, address)
    }
}
